//+------------------------------------------------------------------+
//|                                       Sample_simpleMa_Bunsan.mq4 |
//|                                           copyright(c) Project V |
//|                                http://projectv.hunterchance.com/ |
//+------------------------------------------------------------------+
#property copyright "(c)Project V"
#property link      "http://projectv.hunterchance.com/"

#include <stdlib.mqh>


extern string  _Entry_Setting    = "----------Entry Setting";
extern double  Lots              = 0.01;
extern int     SpaceHour         = 8;
extern int     EntryCount        = 10;
extern int     TakeProfit        = 0;
extern int     StopLoss          = 0;
extern int     Slippage          = 3;
extern int     MagicNumber       = 20101009;

extern string  _MA_Setting       = "----------Moving Average Setting";
extern int     MA1_Period        = 10;
extern int     MA1_Shift         = 0;
extern int     MA1_Method        = MODE_SMA;
extern int     MA1_AppliedPrice  = PRICE_CLOSE;
extern int     MA2_Period        = 21;
extern int     MA2_Shift         = 0;
extern int     MA2_Method        = MODE_SMA;
extern int     MA2_AppliedPrice  = PRICE_CLOSE;

//memo
int  stoplevel;
int  spaceSecond;
bool system_error;
int  closeAllMode;
int  lastsign;
int  entry_cnt;
datetime lastTime;

//+------------------------------------------------------------------+
//| expert initialization function                                   |
//+------------------------------------------------------------------+
int init()
{
   //parameter initialize
   closeAllMode = -1;
   if(!IsTesting())
   {
      if(GlobalVariableCheck("entry_cnt" + "-" + DoubleToStr(MagicNumber, 0)))
      {
         entry_cnt = GlobalVariableGet("entry_cnt" + "-" + DoubleToStr(MagicNumber, 0));
      }
      if(GlobalVariableCheck("lastTime" + "-" + DoubleToStr(MagicNumber, 0)))
      {
         lastTime = GlobalVariableGet("lastTime" + "-" + DoubleToStr(MagicNumber, 0));
      }
   }
   
   //digits
   if(Digits == 3 || Digits == 5)
   {
      TakeProfit *= 10;
      StopLoss   *= 10;
      Slippage   *= 10;
   }
   
   //stop level
   stoplevel = MarketInfo(Symbol(), MODE_STOPLEVEL);
   
   //time space
   spaceSecond = SpaceHour * 60 * 60;
   
   //chart error
   system_error = false;
   string error_comment;
   if(Period() != PERIOD_D1)
   {
      error_comment = "This EA works On Daily chart.";
      Comment("\n", error_comment);
      Alert(error_comment);
      system_error = true;
   }
   if(StringSubstr(Symbol(), 0, 6) != "EURUSD")
   {
      error_comment = "This EA works On EURUSD chart.";
      Comment("\n", error_comment);
      Alert(error_comment);
      system_error = true;
   }
   return(0);
}

//+------------------------------------------------------------------+
//| expert deinitialization function                                 |
//+------------------------------------------------------------------+
int deinit()
{
   if(!IsTesting())
   {
      GlobalVariableSet("entry_cnt" + "-" + DoubleToStr(MagicNumber, 0), entry_cnt);
      GlobalVariableSet("lastTime"  + "-" + DoubleToStr(MagicNumber, 0), lastTime);
   }
   return(0);
}
  
//+------------------------------------------------------------------+
//| expert start function                                            |
//+------------------------------------------------------------------+
int start()
{
   //error
   if(system_error) return(0);
   
   //sign
   static int ma_calc_bar;
   static int sign;
   if(ma_calc_bar != Bars)
   {
      double ma1 = iMA(NULL, 0, MA1_Period, MA1_Shift, MA1_Method, MA1_AppliedPrice, 1);
      double ma2 = iMA(NULL, 0, MA2_Period, MA2_Shift, MA2_Method, MA2_AppliedPrice, 1);
      
      sign = 0;
      if(ma1 > ma2) sign = 1;
      if(ma1 < ma2) sign = -1;
      ma_calc_bar = Bars;
   }
   
   //order
   //exit
   if(sign != 0)
   {
      if(lastsign != sign)
      {
         if(sign == 1) closeAllMode = OP_SELL;
         else if(sign == -1) closeAllMode = OP_BUY;
         
      }
   }
   //close all
   if(closeAllMode >= OP_BUY)
   {
      if(!CloseAll(closeAllMode)) return(0);
      else
      {
         closeAllMode = -1;
         lastsign     = sign;
         lastTime     = 0;
         entry_cnt    = 0;
      }
   }
   
   //entry
   datetime now    = TimeCurrent();
   int      ticket = -1;
   double   sl, tp;
   
   if(entry_cnt < EntryCount)
   {
      if(lastTime + spaceSecond <= now)
      {
         if(lastsign == 1)
         {
            if(StopLoss   > 0) sl = MathMin(Bid - stoplevel * Point, Ask - StopLoss   * Point);
            if(TakeProfit > 0) tp = MathMax(Bid + stoplevel * Point, Ask + TakeProfit * Point);
            ticket = OrderSend(Symbol(), OP_BUY, Lots, Ask, Slippage, sl, tp, NULL, MagicNumber, 0, Blue);
            if(ticket >= 0)
            {
               lastTime = now;
               entry_cnt++;
            }
            else FuncPrintError("Buy");
         }
         else if(lastsign == -1)
         {
            if(StopLoss   > 0) sl = MathMax(Ask + stoplevel * Point, Bid + StopLoss   * Point);
            if(TakeProfit > 0) tp = MathMin(Ask - stoplevel * Point, Bid - TakeProfit * Point);
            ticket = OrderSend(Symbol(), OP_SELL, Lots, Bid, Slippage, sl, tp, NULL, MagicNumber, 0, Red);
            if(ticket >= 0)
            {
               lastTime = now;
               entry_cnt++;
            }
            else FuncPrintError("Sell");
         }
      }
   }
   return(0);
}
//+------------------------------------------------------------------+

//S
bool CloseAll(int type)
{
   for (int i = OrdersTotal() - 1; i >= 0; i--) 
   {
      if(!OrderSelect(i, SELECT_BY_POS)) continue;
      if(OrderSymbol() != Symbol()) continue;
      if(OrderMagicNumber() != MagicNumber) continue;
      if(OrderType() != type) continue;
      if(!OrderClose(OrderTicket(), OrderLots(), OrderClosePrice(), Slippage,Yellow))
      {
         FuncPrintError("Close");
         closeAllMode = type;
         return(false);
      }
   }
   return(true);
}

//Erroro
void FuncPrintError(string ope)
{
   int error = GetLastError();
   Print(ope," operation failed - error(", error, "): ", ErrorDescription(error));
}